<?php

namespace App\Http\Controllers\CMS;

use App\Http\Controllers\Controller;
use App\Libraries\Status;
use App\Models\Admin;
use App\Models\AdminPermission;
use App\Repositories\AdminCategoryRelationshipRepository;
use App\Repositories\AdminRepository;
use App\Services\Common\AdminService;
use App\ViewModels\CMS\AdminPermission\CreateViewModel;
use App\ViewModels\CMS\AdminPermission\EditViewModel;
use App\ViewModels\CMS\AdminPermission\IndexViewModel;
use Dflydev\DotAccessData\Data;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Gate;

class AdminPermissionController extends Controller
{
    public function index(Request $request)
    {
        Gate::authorize("adminPermissionViewAny", AdminPermission::class);

        $admins = AdminService::getAdmins()["data"]["admins"];


        $templateViewModel = new  IndexViewModel(
            $request,
            $admins,
            $request->tab
        );
        return view("cms.admin-permission.index", $templateViewModel);
    }

    public function create(Request $request)
    {
        Gate::authorize("adminPermissionCreate", AdminPermission::class);

        $viewModel = new  EditViewModel(
            $request,
            null,
            "insert",
            null
        );

        return view("cms.admin-permission.create", $viewModel);
    }

    public function store(Request $request)
    {
        Gate::authorize("adminPermissionCreate", AdminPermission::class);

        $result = AdminService::insert(
            $request->name,
            $request->account,
            $request->password,
            $request->status,
            3
        );

        if (Status::isFails($result)) {
            return redirect()->back()
                ->with(Status::bladeFails($result["message"]));
        }

        AdminService::insertPermissions($result['data']['admin']['id'], $request->permissions);


        return redirect()->route("cms.adminPermission.index")
            ->with(Status::bladeSuccess("新增成功"));
    }

    public function edit(Request $request, string $adminId)
    {
        Gate::authorize("adminPermissionUpdate", AdminPermission::class);

        // 一般管理員，無法編輯超級管理員與營運管理員
        if (AdminCategoryRelationshipRepository::isGeneralAdmin(Auth::guard("cms")->id())) {
            if (in_array($adminId, [1, 2])) {
                abort(403);
            }
        } // 營運管理員，無法編輯超級管理員
        elseif (AdminCategoryRelationshipRepository::isOperationAdmin(Auth::guard("cms")->id())) {
            if (in_array($adminId, [1])) {
                abort(403);
            }
        }

        $result = AdminService::getAdmin($adminId);

        $templateViewModel = new  EditViewModel(
            $request,
            $result["data"]["admin"],
            "edit",
            null
        );
        return view("cms.admin-permission.create", $templateViewModel);
    }

    public function update(Request $request, $adminId)
    {
        Gate::authorize("adminPermissionUpdate", AdminPermission::class);

        $result = AdminService::update(
            $adminId,
            $request->name,
            $request->account,
            $request->password,
            $request->status,
            $request->permissions
        );

        return redirect()->back()
            ->with(Status::bladeSuccess("更新成功"));

        // return redirect()->route("cms.adminPermission.index")
        //     ->with(Status::bladeSuccess("更新成功"));
    }

    public function destroy(Request $request, Admin $admin)
    {
        Gate::authorize("adminPermissionDelete", AdminPermission::class);

        $admin->delete();

        return redirect()->route("cms.adminPermission.index")
            ->with(Status::bladeSuccess("刪除成功"));
    }
}
