<?php

namespace App\Http\Controllers\CMS;

use App\Http\Controllers\Controller;
use App\Libraries\Status;
use App\Models\Order;
use App\ViewModels\CMS\Dashboard\IndexViewModel;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\File;

class DashboardController extends Controller
{
    public function index(Request $request)
    {
        $categoryId = Auth::guard("cms")->user()->load("categories")->categories->first()->id;

        if (in_array($categoryId, [2, 3])) {
            // 是否強制 3 跳轉到最新消息 (形象網站適合)
            if ($categoryId === 3) {
                return redirect()->route("cms.news.index");
            }

            // 如果沒有權限觀看訂單
            if ($request->user()->can("orderViewAny", Order::class) === false) {
                return view("cms.dashboard.emptyIndex");
            }

            // 跳轉到訂單
            return redirect()->route("cms.news.index");
        }

        // 系統工程師在看的介面
        $robots = $this->getRobots();
        $viewModel = new IndexViewModel($robots);
        return view("cms.dashboard.index", $viewModel);
    }

    private function getRobots()
    {
        $robotsFilePath = public_path('robots.txt');
        if (File::exists($robotsFilePath)) {
            return File::get($robotsFilePath);
        }
        return "";
    }
}
