<?php

namespace App\Http\Controllers\CMS;

use App\Http\Controllers\Controller;
use App\Libraries\Status;
use App\Services\Common\GoogleReCaptchaService;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\Auth;

class LoginController extends Controller
{
    public function index(Request $request)
    {
        if (Auth::guard('cms')->check()) {
            return redirect()->route("cms.dashboard.index");
        }

        return view("cms.login.index");
    }

    public function update(Request $request)
    {
        $credentials = $request->only(['account', 'password']);

        // 正式環境將採用 Google ReCaptcha 驗證
        if (App::environment('production')) {
            $result = GoogleReCaptchaService::validate(
                $request->input('g-recaptcha-response'),
                $request->ip()
            );

            if (Status::isFails($result)) {
                return redirect()->route("cms.login.index")
                    ->with(Status::bladeFails("驗證失敗"));
            }
        }

        $attempt = Auth::guard('cms')
            ->attempt($credentials, true);

        if ($attempt === false) {
            return redirect()->route("cms.login.index")
                ->with(Status::bladeFails("帳號密碼輸入錯誤"));
        }

        $request->session()->regenerate();

        return redirect()->route("cms.dashboard.index");
    }
}
