<?php

namespace App\Http\Controllers\CMS;

use App\Http\Controllers\Controller;
use App\Libraries\Status;
use App\Models\Admin;
use App\Models\Membership;
use App\Models\User;
use App\Services\CMS\UserMembershipService;
use App\Services\CMS\UserService;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Gate;

class MembershipController extends Controller
{
    public function store(Request $request)
    {
        Gate::authorize("adminCreate", Admin::class);

        UserMembershipService::insertMembership(
            $request->userId,
            $request->startAt,
            $request->endAt
        );

        return redirect()->away($request->redirect)
            ->with(Status::bladeSuccess("新增 VIP 成功，並觸發送出贈品"));
    }

    /**
     * 目前的贈品未處理收回的行為，需要手動執行
     *
     * @param  Membership  $membership
     *
     * @return RedirectResponse
     */
    public function destroy(Membership $membership)
    {
        Gate::authorize("adminDelete", Admin::class);

        UserMembershipService::deleteVIP($membership);

        return redirect()->back()->with(Status::bladeSuccess("刪除成功"));
    }
}
