<?php

namespace App\Http\Controllers\CMS;

use App\Models\PostCategory;
use App\Repositories\PostCategoryRepository;
use App\Services\CMS\PostCategoryService;
use App\ViewModels\CMS\NewsCategory\EditViewModel;
use App\ViewModels\CMS\NewsCategory\IndexViewModel;
use App\Http\Controllers\Controller;
use App\Libraries\Status;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Gate;

class NewsCategoryController extends Controller
{
    public function index(Request $request)
    {
        Gate::authorize("newsCategoryViewAny", PostCategory::class);

        $result = PostCategoryService::getCategoriesByParentId(100);

        $indexViewModel = new IndexViewModel(
            $request,
            $result['data']['categories'],
            $request->tab
        );
        return view("cms.news-category.index", $indexViewModel);
    }

    public function create(Request $request)
    {
        Gate::authorize("newsCategoryCreate", PostCategory::class);

        return view("cms.news-category.create");
    }

    public function store(Request $request)
    {
        Gate::authorize("newsCategoryCreate", PostCategory::class);

        $result = PostCategoryService::insert($request->title, $request->parentId);

        return redirect()->route("cms.newsCategory.index")
            ->with(Status::bladeSuccess("新增成功"));
    }

    public function edit(Request $request, string $id)
    {
        Gate::authorize("newsCategoryUpdate", PostCategory::class);

        $result = PostCategoryService::getCategory($id);

        if (Status::isFails($result)) {
            return redirect()->route("cms.newsCategory.index")->with(
                Status::bladeFails($result['message'])
            );
        }


        $NewsCategoryViewModel = new  EditViewModel($request, $result['data']['category'], null);
        return view("cms.news-category.edit", $NewsCategoryViewModel);
    }

    public function update(Request $request)
    {
        Gate::authorize("newsCategoryUpdate", PostCategory::class);

        // 存在嗎
        $result = PostCategoryService::update($request->title, $request->id);

        if (Status::isFails($result)) {
            return redirect()->route("cms.newsCategory.index")
                ->with(Status::bladeFails($result['message']));
        }

        return redirect()->route("cms.newsCategory.index")
            ->with(Status::bladeSuccess("修改成功"));
    }

    public function destroy(Request $request)
    {
        Gate::authorize("newsCategoryDelete", PostCategory::class);

        // 如果底下有文章，不予刪除
        $result = PostCategoryService::delete($request->id);

        if (Status::isFails($result)) {
            return redirect()->route("cms.newsCategory.index")
                ->with(Status::bladeFails($result['message']));
        }

        return redirect()->route("cms.newsCategory.index")
            ->with(Status::bladeSuccess("成功刪除 {$result['data']['num']} 筆"));
    }
}
