<?php

namespace App\Http\Controllers\CMS;

use App\Http\Controllers\Controller;
use App\Libraries\Status;
use App\Models\Form;
use App\Models\SettingReservation;
use Illuminate\Contracts\View\Factory;
use Illuminate\Contracts\View\View;
use Illuminate\Foundation\Application;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Gate;

class ReservationController extends Controller
{
    public function index()
    {
        Gate::authorize("reservationViewAny", Form::class);

        // 在這裡處理後台預約系統的顯示
        $settingReservations = SettingReservation::all();

        if ($settingReservations->isEmpty()) {
            return redirect()->route('cms.settingReservation.create');
        }

        return redirect()->route('cms.settingReservation.edit', [
            'settingReservation' => $settingReservations->first(),
        ]);
    }

    public function create()
    {
        Gate::authorize("reservationCreate", Form::class);

        // 在這裡處理後台預約系統的建立表單
        return view('cms.setting-reservation.create');
    }

    public function store(Request $request)
    {
        Gate::authorize("reservationCreate", Form::class);

        // 在這裡處理儲存預約資料的邏輯
        $data = $request->validate([
            'start_date' => 'required|date',
            'end_date' => 'required|date|after_or_equal:start_date',
            'daily_slots' => 'array', // 這裡可以加上更多的驗證規則，例如檢查每個時段是否合法
        ]);

        $settingReservation = SettingReservation::query()->create($data);

        return redirect()->route('cms.settingReservation.edit', [
            'settingReservation' => $settingReservation,
        ])->with(Status::bladeSuccess('預約系統建立成功'));
    }

    /**
     * 這個用法可以參考
     * https://laravel.com/docs/10.x/routing#implicit-binding
     *
     * @param  SettingReservation  $settingReservation
     *
     * @return \Illuminate\Contracts\Foundation\Application|Factory|View|Application
     */
    public function edit(SettingReservation $settingReservation)
    {
        Gate::authorize("reservationUpdate", Form::class);

        // 在這裡處理後台預約系統的編輯表單
        return view('cms.setting-reservation.edit',
            compact('settingReservation'));
    }

    public function update(
        Request $request,
        SettingReservation $settingReservation
    ) {
        Gate::authorize("reservationUpdate", Form::class);

        // 在這裡處理更新預約資料的邏輯
        $data = $request->validate([
            'start_date' => 'required|date',
            'end_date' => 'required|date|after_or_equal:start_date',
            'daily_slots' => 'array', // 這裡可以加上更多的驗證規則，例如檢查每個時段是否合法
        ]);

        $settingReservation->update($data);

        return redirect()->route('cms.settingReservation.edit', [
            'settingReservation' => $settingReservation,
        ])->with(Status::bladeSuccess('預約系統更新成功'));
    }
}
