<?php

namespace App\Http\Controllers\CMS;

use App\Http\Controllers\Controller;
use App\Libraries\Status;
use App\Models\Setting;
use App\Repositories\AdminCategoryRelationshipRepository;
use App\Repositories\AdminPermissionRepository;
use App\Repositories\SettingRepository;
use App\Services\Common\AdminService;
use App\ViewModels\CMS\Setting\EditViewModel;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Gate;

class SettingController extends Controller
{
    public function edit(?string $currentGroup = null)
    {
        if ($currentGroup === null) {
            return redirect()->route("cms.setting.edit", ["group" => "companyInfo"]);
        }

        Gate::authorize("settingUpdate", Setting::class);

        if (AdminCategoryRelationshipRepository::isGeneralAdmin(Auth::guard("cms")->id()) and $currentGroup === "system") {
            // abort(403);
        }


        // 希望的排序
        $sort = ["socialMedia", "pageBlock", "coupons", "vip",];

        // 取得所有群組
        $groups = SettingRepository::getGroups()->pluck("group");

        // // 如果是一般管理員，移除 system
        // if (AdminCategoryRelationshipRepository::isGeneralAdmin(Auth::guard("cms")->id())) {
        //     $key = $groups->search("system");
        //     $groups->forget($key);
        // }

        // 可指定要移除的群組
        $removeGroups = $this->getRemoveGroups();

        if (in_array($currentGroup, $removeGroups)) {
            abort(403);
        }

        collect($removeGroups)->each(function ($item) use (&$groups) {
            $key = $groups->search($item);
            $groups->forget($key);
        })->values();


        $groups = $groups->sortBy(function ($item, $key) use ($sort) {
            return array_search($item, $sort);
        })->values();

        // 要顯示的列表
        $settings = SettingRepository::getSettingsByGroup($currentGroup);

        $viewModel = new EditViewModel($groups, $currentGroup, $settings);
        return view('cms.setting.edit2', $viewModel);
    }

    public function update(Request $request, Setting $setting)
    {
        Gate::authorize("settingUpdate", Setting::class);

        $request->validate([
            'value' => 'required',
        ]);

        $setting->value = $request->value;
        $setting->save();

        return redirect()->back()->with(Status::bladeSuccess("更新成功"));
    }

    /**
     * 可指定要移除的群組
     * @return string[]
     */
    public function getRemoveGroups(): array
    {
        if (AdminCategoryRelationshipRepository::isGeneralAdmin(Auth::guard("cms")->id())) {
            return ["vip", "coupons", 'logistics', 'shipping', 'pageBlock'];
        } elseif (AdminCategoryRelationshipRepository::isOperationAdmin(Auth::guard("cms")->id())) {
            return ["vip", "coupons", 'logistics', 'shipping', 'pageBlock'];
        } // 超級管理員都顯示
        else {
            return [];
        }
    }
}
