<?php

namespace App\Http\Controllers\CMS;

use App\Libraries\Status;
use App\Models\Admin;
use App\Models\User;
use App\Services\CMS\CouponService;
use App\ViewModels\CMS\User\EditViewModel;
use App\Http\Controllers\Controller;
use App\Libraries\ErrorMessage;
use App\Services\CMS\UserService;
use App\ViewModels\CMS\User\IndexViewModel;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Gate;

class UserController extends Controller
{
    public function index(Request $request)
    {
        Gate::authorize("adminViewAny", Admin::class);

        $users = UserService::getUserPaginates(
            ['*'],
            $request->searchType,
            $request->keyword,
            $request->membership
        );

        $userViewModel = new IndexViewModel(
            $request,
            $users,
            $request->tab
        );

        return view("cms.users.index", $userViewModel);
    }

    /**
     * 切換前台會員身份，用來除錯或是了解該名會員登入後的狀況
     * 請小心使用！並需要經過該用戶授權同意
     *
     * @param  Request  $request
     * @param  User  $user
     *
     * @return RedirectResponse
     */
    public function switchIndex(Request $request, User $user)
    {
        Gate::authorize("adminViewAny", Admin::class);

        Auth::guard("web")->login($user, true);

        return redirect()->route("foreground.profile.index")
            ->with(Status::bladeSuccess("成功切換身份"));
    }

    public function edit(Request $request, string $id)
    {
        Gate::authorize("adminUpdate", Admin::class);

        $user = UserService::getUserWithTrashedMemberships($id);
        if (ErrorMessage::isExists($user)) {
            abort(404);
        }

        $couponHistoryResult = CouponService::getUserCouponHistories($user);

        $userViewModel = new EditViewModel($request, $user, $couponHistoryResult['data']['coupons']);
        return view("cms.users.edit", $userViewModel);
    }

    public function update(Request $request)
    {
        Gate::authorize("adminUpdate", Admin::class);

        $result = UserService::update(
            $request->id,
            $request->name,
            $request->status,
            $request->phone,
            $request->birthday,
            $request->gender,
            $request->email
        );

        if ($result['status'] === "fails") {
            if ($result['code'] === "emailExists") {
                return redirect()->back()
                    ->with(Status::bladeFails($result['message']));
            } else {
                return redirect()->route("cms.users.index")
                    ->with(Status::bladeFails("未成功修改"));
            }
        } else {
            return redirect()->route("cms.users.index")
                ->with(Status::bladeSuccess("修改成功"));
        }
    }

    public function destroy(Request $request)
    {
        Gate::authorize("adminDelete", Admin::class);

        $num = UserService::delete($request->id);

        return redirect()->route("cms.users.index")
            ->with(Status::bladeSuccess("成功刪除 {$num} 筆"));
    }
}
