<?php

namespace App\Http\Controllers\Foreground;

use App\Http\Controllers\Controller;
use App\Libraries\Status;
use App\Services\Common\GoogleReCaptchaService;
use App\Services\Foreground\FormService;
use App\Services\Foreground\SettingService;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Validator;
use ReCaptcha\ReCaptcha;

class ContactController extends Controller
{
    protected array $rules
        = [
            "name" => ["required", "max:30"],
            "email" => ["nullable", "email"],
            "phone" => ["required", "max:20"],
            "message" => ["nullable", "max:500"],
        ];

    protected array $messages
        = [
            'name.required' => '請輸入稱呼',
            'name.max' => '稱呼最多 30 個字',
            'email.email' => 'Email 格式錯誤',
            'phone.required' => '請輸入聯絡電話',
            'phone.max' => '聯絡電話最多 20 個字',
            'message.max' => '備註最多 500 個字',
        ];

    public function create()
    {
        $companyInfo = SettingService::getSettingsByGroup('companyInfo', ['id', 'key', 'value']);
        $tel = $companyInfo->where('key', 'company_tel')->first()->value;
        return view("foreground.contact.create",compact('tel'));
    }

    public function store(Request $request)
    {
        // 驗證欄位
        $make = Validator::make(
            $request->all(),
            $this->rules,
            $this->messages
        );

        if ($make->fails()) {
            Log::warning(
                "聯絡我們欄位驗證失敗",
                $make->errors()->toArray()
            );

            return redirect()->route("foreground.contact.create")
                ->withErrors($make)
                ->withInput();
        }

        // 驗證 Google reCaptcha
        $result = GoogleReCaptchaService::validate(
            $request->{'g-recaptcha-response'},
            $request->ip()
        );

        if (Status::isFails($result)) {
            Log::info(
                "聯絡我們 reCaptcha 未通過驗證",
                $result['data']['errors']
            );
            return redirect()->route("foreground.contact.create")->with(
                Status::bladeFails("您未通過 Google reCaptcha 驗證，請重新操作")
            )->withInput();
        }

        $message = $request->question." ".$request->message;
        // 儲存聯絡表單
        FormService::insertContact(
            $request->name,
            $request->email,
            $request->phone,
            $message
        );

        return redirect()->route("foreground.contact.resultIndex")->withInput();
    }

    public function resultIndex(Request $request)
    {
        // dd($request->old());
        if (empty($request->old("name"))) {
            return redirect()->route("foreground.contact.create");
        }

        return view("foreground.contact.result-index", compact('request'));
    }
}
