<?php

namespace App\Http\Controllers\Foreground;

use App\Enums\CacheKey;
use App\Enums\CacheTime;
use App\Http\Controllers\Controller;
use App\Libraries\CacheGenerate;
use App\Models\Setting;
use App\Repositories\PostRepository;
use App\Repositories\ProductCategoryRepository;
use App\Repositories\ProductRepository;
use App\Services\CMS\PostService;
use App\Services\CMS\ProductService;
use App\Services\Foreground\BannerService;
use App\Services\Foreground\PostCategoryService;
use App\Services\Foreground\ProductCategoryService;
use App\Services\Foreground\SettingService;
use App\ViewModels\Foreground\Home\IndexViewModel;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Cache;

class HomeController extends Controller
{
    public function index()
    {
        // 頁面區塊
        $pageBlock = SettingService::getSettingsByGroup("pageBlock", ["key", "title", "value"]);

        // Banner
        $banner = PostRepository::getPostById(1, null, "imageRelationship", ["*"]);

        // 最新消息
        $newsResult = \App\Services\Foreground\PostService::getIndexPage(100, 6, null, null);

        // 熱銷商品
        $products = ProductRepository::getPaginateByProductCategoryIds(
            [6],
            24,
            ['*'],
            false,
            null
        );

        // 文章專區
        $articleResult = \App\Services\Foreground\PostService::getIndexPage(200, 3, null, null);

        // 相簿專區
        $albumResult = \App\Services\Foreground\PostService::getIndexPage(300, 8, null, null);

        $about = \App\Services\Foreground\PostService::getIndexPage(500, 8, null, null)['posts']->where(
            'id',
            '1000'
        )->first();

        // // 開發請打開這個可以方便快速查閱
        // dump([
        //     "頁面區塊" => $pageBlock->toArray(),
        //     "Banner" => $banner->toArray(),
        //     "最新消息" => collect($newsResult['posts']->items())->toArray(),
        //     //     "產品分類" => $productCategories->toArray(),
        //     "商品資訊" => $products->items()[0]->toArray(),
        //     "相簿專區" => collect($albumResult['posts']->items())->toArray(),
        //     "文章專區" => collect($articleResult['posts']->items())->toArray(),
        //     //     "房詐騙宣導" => $fraudPreventionWarning['data']['post'],
        // ]);

        // ------- 以下並不顯示在頁面但是予以保留，提供參考或未來使用 -------
        // 商品分類
        $productCategoryIdsResult = ProductCategoryService::getDescendentsByProductCategoryId(1);
        $productCategories = $productCategoryIdsResult['data']['descendents'];

        // 取得指定產品分類和全部分類底下的所有產品分類編號
        $lastProductCategoryIds = ProductCategoryRepository::getLastProductCategoryIds(
            rootProductCategoryId: "1",
            isPluckIds: true
        );

        // 防詐騙宣導
        $fraudPreventionWarning = PostService::getPost(5, "images");


        // 依照首頁服務項目的版面來排序資料
        $customSort = [
            '5',
            '9',
            '10',
            '6',
            '7',
            '8',
            '11',
        ];
        $albums = collect();
        foreach ($customSort as $sort) {
            foreach ($albumResult['posts'] as $album){
                if($sort == $album->id){
                    $albums->push($album);
                }
            }
        }

        $viewModel = new IndexViewModel(
            $pageBlock,
            $banner,
            $newsResult['posts'],
            $productCategories,
            $products,
            $albums,
            $articleResult['posts'],
            $fraudPreventionWarning['data']['post'],
            $about
        );
        return view("foreground.home.index", $viewModel);


        // ---- 以下之後刪除 ----

        // $key = CacheGenerate::website(
        //     CacheKey::HomeIndex,
        // );
        // $time = CacheTime::HomeIndex;
        // $result = Cache::remember($key, $time, function () {
        //     $banner = PostRepository::getPostById(1, null, ["*"]);
        //
        //     $productFeatures = PostRepository::getPostById(2, null, ["*"]);
        //
        //     $notice = SettingService::getSettingByKey("fraudPreventionWarning");
        //
        //     return compact("banner", "productFeatures", "notice");
        // });
        //
        // // 內部有許多變數改用 livewire 結構
        // return view("foreground.home.index", [
        //     'banner' => $result['banner'],
        //     'productFeatures' => $result['productFeatures'],
        //     'notice' => $result['notice'],
        // ]);
    }

    private function getProducts()
    {
        // 取得商品專區分類和全部分類底下的所有產品
        $productCategoryIds = empty($categoryId)
            ? ProductCategoryRepository::getLastProductCategoryIds(
                1,
                true
            )
            : [$categoryId];


        $products = ProductRepository::getPaginateByProductCategoryIds(
            $productCategoryIds,
            8,
            ['*'],
            true,
            null
        );

        return $products;
    }


    private function getNews()
    {
        // 取得分類底下的所有文章和分頁
        $aryCategoryId = PostCategoryService::getRootCategoriesByParentId(
            1,
            null,
            ['id', 'title']
        );
        return $posts = PostRepository::getPostsByPaginate(['*'],
            6,
            $aryCategoryId,
            []);
    }
}
