<?php

namespace App\Http\Controllers\Foreground;

use App\Enums\CacheKey;
use App\Enums\CacheTime;
use App\Http\Controllers\Controller;
use App\Libraries\CacheGenerate;
use App\Repositories\ImageRepository;
use Exception;
use Illuminate\Http\RedirectResponse;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\Storage;
use Intervention\Image\Facades\Image;
use Intervention\Image\ImageCache;
use League\CommonMark\Extension\CommonMark\Node\Inline\Strong;
use function PHPUnit\Framework\lessThanOrEqual;

class ImageController extends Controller
{
    /**
     * 如果要一張編號 777 的圖片長寬最大 1200px，在 blade 中可以使用
     * <img src="{{ route("foreground.image.show", [777, 1200, 1200]) }}" alt="image">
     *
     * @param $id
     * @param $width
     * @param $height
     * @return RedirectResponse|\Intervention\Image\Image
     * @throws Exception
     */
    public function show($id, $width, $height)
    {
        $driverName = "publicOnPublic";

        if ($driverName === "public") {
            return $this->generateResize($id, $width, $height);
        } elseif ($driverName === "publicOnPublic") {
            // return $this->loadImageContent($id);
            return $this->redirectImage($id);
        } else {
            throw new Exception("不支援的圖片顯示驅動");
        }
    }

    /**
     * @param $id
     * @param $width
     * @param $height
     * @return mixed
     * @deprecated 這個方法不再使用，因為原本的套件不再維護了 {@see https://packagist.org/packages/intervention/imagecache}
     */
    private function generateResize($id, $width, $height)
    {
        return Image::cache(function (ImageCache $image) use ($id, $width, $height) {
            $key = CacheGenerate::website(CacheKey::ImageShow, $id, $width, $height);
            $time = CacheTime::ImageShow;
            $model = Cache::remember(
                $key,
                $time,
                function () use ($id) {
                    return ImageRepository::getImageById($id);
                }
            );


            if ($model === null) {
                abort(403, "請求的圖片資源錯誤");
            }


            // 圖片不存在
            $realpath = base_path("storage/app/public/{$model->file_original}");
            if (file_exists($realpath) === false) {
                abort(403, "請求的圖片資源錯誤");
            }

            $greyscale = $image->make($realpath)->resize($width, $height, function ($constraint) {
                $constraint->aspectRatio();
                $constraint->upsize();
            })->encode("webp", 95);
        }, 60);
    }

    private function loadImageContent($id)
    {
        $result = \App\Models\Image::find($id);
        if ($result === null) {
            abort(403, "請求的圖片資源錯誤");
        }

        $realpath = realpath("storage/{$result->file_m}");
        return file_get_contents($realpath);
    }

    private function redirectImage($id)
    {
        $result = \App\Models\Image::find($id);
        if ($result === null) {
            abort(403, "請求的圖片資源錯誤");
        }

        $url = url("storage/{$result->file_m}");
        return redirect()->away($url);
    }
}
