<?php

namespace App\Http\Controllers\Foreground;

use App\Http\Controllers\Controller;
use App\Services\Foreground\PostCategoryService;
use App\Services\Foreground\PostService;
use App\ViewModels\Foreground\Knowledge\IndexViewModel;
use App\ViewModels\Foreground\Knowledge\ShowViewModel;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;

class KnowledgeController extends Controller
{
    protected array $rules = [
        'keyword' => 'nullable|string|min:2'
    ];

    protected array $messages = [
        'keyword.min' => '關鍵字至少兩個字，才能有效縮小範圍搜尋'
    ];

    public function index(Request $request)
    {
        Validator::make([
            'keyword' => $request->keyword
        ], $this->rules, $this->messages)->validate();

        $data = PostService::getIndexPage(
            rootParentId: 200,
            postLimit: 16,
            categoryId: $request->categoryId ?? null,
            keyword: $request->keyword ?? null
        );
        $indexViewModel = new IndexViewModel(
            $request,
            $data['postCategories'],
            $data['posts']
        );

        return view('foreground.knowledge.index', $indexViewModel);
    }

    public function show($id)
    {
        // 分類務必再根分類底下，避免如最新消息的編號輸入在知識學堂
        $result = PostService::getShowPage($id, rootPostCategoryId: 200);
        if ($result['post'] === null) {
            abort(404);
        }

        $showViewModel = new ShowViewModel(
            $result['post'],
            $result['otherPosts']
        );

        return view('foreground.knowledge.show', $showViewModel);
    }
}
