<?php

namespace App\Http\Controllers\Foreground;

use App\Http\Controllers\Controller;
use App\Libraries\Cart;
use App\Libraries\CartChannel;
use App\Libraries\Status;
use App\Services\Foreground\CartService;
use Dflydev\DotAccessData\Data;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;

class LoginController extends Controller
{
    protected $rules = [
        'account' => ['required'],
        'password' => [
            'required',
            'min:6',
            'regex: /^(?=.*[a-zA-Z])(?=.*\d).+$/'
        ]
    ];

    protected $messages = [
        'account.required' => '請輸入 Email 或手機',
        'password.regex' => '請輸入6位(含)以上英文和數字組合密碼',
        'password.required' => '請填寫密碼',
        'password.min' => '密碼最少須要 6 位數',
    ];

    public function index(Request $request)
    {
        if (Auth::guard("web")->check()) {
            return redirect()->route("foreground.profile.index");
        }

        $redirect = $request->redirect ?? null;
        return view('foreground.login.index', compact('redirect'));
    }

    public function store(Request $request)
    {
        $data = [
            'account' => $request->account,
            'password' => $request->password,
            'source' => "website"
        ];

        Validator::make($data, $this->rules, $this->messages)->validate();

        // 帳號登入同時比對 email 或 phone 其中一個對了就可以
        $attempt = Auth::guard("web")->attempt([
            'email' => $data['account'],
            'password' => $data['password'],
            'source' => $data['source']
        ]);
        if ($attempt === false) {
            $attempt = Auth::guard("web")->attempt([
                'phone' => $data['account'],
                'password' => $data['password'],
                'source' => $data['source']
            ]);
        }

        if ($attempt === false) {
            return redirect()->route("foreground.login.index")
                ->with(Status::bladeFails("帳號密碼輸入錯誤"));
        }

        $request->session()->regenerate();

        CartService::restore();

        if (!empty($request->redirect)) {
            return redirect()->away($request->redirect);
        }
        return redirect()->route("foreground.profile.index");
    }


}
