<?php

namespace App\Http\Controllers\Foreground;

use App\Http\Controllers\Controller;
use App\Libraries\Status;
use App\Services\Foreground\CartService;
use App\Services\Foreground\LoginWithSocialiteService;
use App\Services\Foreground\UserService;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Log;
use Laravel\Socialite\Facades\Socialite;
use Laravel\Socialite\Two\InvalidStateException;
use Psr\Container\ContainerExceptionInterface;
use Psr\Container\NotFoundExceptionInterface;

class LoginWithLineController extends Controller
{
    public function index(Request $request)
    {
        // 如果有傳入 redirect 參數，就記錄到 session 中
        if (!empty($request->redirect)) {
            session(['redirect' => $request->redirect]);
        }


        return Socialite::driver('line')->redirect();
    }

    public function callbackIndex(Request $request)
    {
        try {
            $user = Socialite::driver('line')->user();
            LoginWithSocialiteService::rememberToSession($user);
            return redirect()->route("foreground.loginWithLine.checkExistsIndex");
        } catch (InvalidStateException $e) {
            abort(422, "可能是來源瀏覽器網址與系統網址比對不符合。");
        }
    }

    public function checkExistsIndex()
    {
        $data = session()->get("loginWithSocialite");
        $user = $data->user;

        // 取得內部會員資料，代表檢查曾經註冊過嗎
        $result = UserService::getUserBySocialite(
            $user['sub'],
            "line"
        );

        // 準備要登入的會員
        $loginUser = null;

        // 用來追蹤使用
        $channel = null;

        if (empty($user['email'])) {
            return redirect()->route("foreground.login.index")
                ->with(
                    Status::bladeFails(
                        "您必須授權 Line 的 Email"
                    )
                );
        }

        // 已經存在資料表中
        if (Status::isSuccess($result)) {
            $loginUser = $result['data']['user'];
            $channel = "old";
        } // 不存在就註冊
        elseif (Status::isFails($result, "notExists")) {
            $result = UserService::createSimple(
                $user['email'],
                "line",
                null,
                $user['name'],
                null,
                null,
                null,
                null,
                $user['sub'],
                $user['picture']
            );

            // 如果 Email 被使用過
            if (Status::isFails($result, "emailExists")) {
                return redirect()->route("foreground.login.index")
                    ->with(
                        Status::bladeFails("Email 已被使用，請改用其他帳號登入")
                    );
            } // 成功新增會員，因此直接登入
            elseif (Status::isSuccess($result, "insertUser")) {
                $loginUser = $result['data']['user'];
                $channel = "new";
            } else {
                return redirect()->route("foreground.login.index")
                    ->with(
                        Status::bladeFails(
                            "發生預期之外的錯誤，請與我們聯繫 100"
                        )
                    );
            }
        } else {
            return redirect()->route("foreground.login.index")
                ->with(
                    Status::bladeFails("發生預期之外的錯誤，請與我們聯繫 200")
                );
        }

        session()->flash("loginUser", $loginUser);
        session()->flash("channel", $channel);

        return redirect()->route("foreground.loginWithLine.finish");
    }

    /**
     * @return RedirectResponse
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function finish()
    {
        $loginUser = session()->get("loginUser");
        $channel = session()->get("channel");

        if (empty($loginUser) and empty($channel)) {
            return redirect()->route("foreground.login.index")
                ->with(Status::bladeFails("操作已失效，請重新登入"));
        }

        Auth::guard("web")->login($loginUser);

        CartService::restore();

        // 如果有傳入 redirect 參數，就導向該網址，並註銷 redirect session
        $redirect = session()->get("redirect");
        if ($redirect) {
            session()->forget("redirect");
            return redirect()->away($redirect);
        }

        return redirect()->route("foreground.profile.index");
    }
}
