<?php

namespace App\Http\Controllers\Foreground;

use App\Http\Controllers\Controller;
use App\Libraries\Shipping;
use App\Libraries\Status;
use App\Libraries\Url;
use App\Models\LogisticsMemory;
use App\Presenters\EcPay\LogisticPresenter;
use App\Repositories\LogisticsMemoryRepository;
use App\Services\Foreground\CartService;
use App\Services\Foreground\EcPay\LogisticService;
use App\Services\Foreground\LogisticsService;
use App\Services\Foreground\SettingService;
use Dflydev\DotAccessData\Data;
use Ecpay\Sdk\Exceptions\RtnException;
use Ecpay\Sdk\Factories\Factory;
use Ecpay\Sdk\Response\AesJsonResponse;
use ErrorException;
use Exception;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Validator;

class LogisticsController extends Controller
{
    public function exampleIndex(Request $request)
    {
        dump($request->session()->all());
        $link = route('foreground.logistics.redirectIndex', [
            'redirect' => url()->full(),
            'userName' => 'Cary',
            'userPhone' => '0978235353'
        ]);
        return "<a href='{$link}'>前往測試</a>";
    }

    /**
     *
     * @link https://developers.ecpay.com.tw/?p=10112
     *
     *
     * @return mixed
     * @throws RtnException
     */
    public function redirectIndex(Request $request)
    {
        $result = Validator::make($request->all(), [
            'redirect' => 'required|url',
            'userName' => 'required',
            'userPhone' => 'required',
        ], [
            'redirect.required' => 'redirect 參數不存在',
            'redirect.url' => 'redirect 參數必須是網址',
            'userName.required' => 'userName 參數不存在',
            'userPhone.required' => 'userPhone 參數不存在',
        ]);

        if ($result->fails()) {
            abort(403, $result->errors()->first());
        }

        // 準備前往綠界物流
        $logisticService = new LogisticService();
        echo $logisticService->getRedirect(
            tempLogisticsID: 0,
            goodsAmount: $request->goodsAmount ?? 100,
            goodsName: $request->goodsAmount ?? config("website.name")." 商品",
            senderName: SettingService::getSettingByKey('EcPayLogisticsSenderName')->value,
            senderZipCode: SettingService::getSettingByKey('EcPayLogisticsSenderZipCode')->value,
            senderAddress: SettingService::getSettingByKey('EcPayLogisticsSenderAddress')->value,
            isCollection: "N",
            remark: $request->remark ?? null,
            temperature: $request->temperature ?? "0001",
            specification: $request->specification ?? "0001",
            scheduledPickupTime: $request->scheduledPickupTime ?? "4",
            receiverAddress: $request->receiverAddress ?? null,
            receiverZipCode: $request->receiverZipCode ?? null,
            receiverCellPhone: $request->userPhone ?? null,
            receiverPhone: $request->userTel ?? null,
            receiverName: $request->userName ?? null,
            enableSelectDeliveryTime: $request->enableSelectDeliveryTime ?? "N",
            eshopMemberID: $request->eshopMemberID ?? null,
            serverReplyURL: route("foreground.logistics.serverReplayIndex"),
            clientReplyURL: route("foreground.logistics.clientReplayIndex", ["redirect" => $request->redirect])
        );

    }

    /**
     * @throws Exception
     */
    public function resultIndex(Request $request, string $cartChannelValue)
    {
        dd("使用 livewire 模式應該就不會來到這裡了");
        // $result = LogisticsService::checkResult($request->tmpKey);
        // if (Status::isFails($result)) {
        //     abort(403, "操作上發生錯誤，網頁已過期: {$result['code']}");
        // }

        $fromEcLogistics = $request->session()->get('fromEcLogistics');

        // 從 session 取回離開網站前紀錄的使用者填寫資料後，並清除
        $formData = $request->session()->get('formData');
        if (empty($formData)) {
            abort(403, "網頁已過期，請返回重新操作。");
        }

        $request->session()->forget('formData');

        // 記錄到運費類別
        $title = LogisticPresenter::getSubType(
            $fromEcLogistics['LogisticsSubType']
        );

        $price = LogisticsService::getPriceFromSetting(
            $fromEcLogistics['LogisticsType']
        );

        (new Shipping())->add($title, $price, false);


        /**
         * 綠界回傳的資料，也把綠界有的個人資料填充到表單，使使用者填寫的資料與綠界同步。
         * 使用者地址表的地址因為雙方格式不同，因此我方不以綠界填充。
         * 訂單的地址，會以綠界回傳的為主
         */
        $forFormData = CartService::setCarEncrypt($fromEcLogistics, $formData);

        // 回到購物車選擇物流的表單
        return redirect()->route("foreground.cart.checkIndex", [
            'cartChannelValue' => $cartChannelValue,
            'hash' => $forFormData
        ]);
    }

    public function serverReplayIndex(Request $request)
    {
        Log::info("綠界物流狀態回傳", $request->all());
    }

    /**
     * 從綠界導向回來
     *
     * 解密方法可以參考
     * @link https://github.com/ECPay/SDK_PHP/blob/master/example/Logistics/AllInOne/TempTradeEstablishedResponse.php
     *
     * @param  Request  $request
     *
     * @return RedirectResponse
     * @throws RtnException
     */
    public function clientReplayIndex(Request $request)
    {
        Log::info("綠界當使用者選擇物流後回傳，解析前", $request->all());

        $logisticService = new LogisticService();
        $decode = $logisticService->getClientReplay($request->ResultData);
        if (Status::isFails($decode)) {
            abort(500, $decode['code']);
        }

        Log::info("綠界當使用者選擇物流後回傳，解析後", $request->all());

        try {
            // 返回原本的位置, 並把必要的參數帶入。過去使用 with() 但是並無法作用，因為這是拋到完整網址，laravel 無法得知是否為外部網址。因此使用 with() 會被忽略。
            // 我想是因為 with 本身透過 session 傳遞，而 session 本身是存在 cookie 中，因此無法跨網域。
            // 因此改為參數帶入 query string

            $pathWithQuery = Url::getPathWithQueryByRoute($request->redirect, [
                'fromEcLogistics' => encrypt($decode['data']['Data']),
            ]);
            $url = url($pathWithQuery);

            return redirect()->away($url);
        } catch (ErrorException $exception) {

            if ($decode['data']['Data']['RtnCode'] === 0) {
                // 返回原本的位置
                return redirect()->away($request->redirect)
                    ->with(Status::bladeFails("交易已逾期"));
            }
        }
    }


}
