<?php

namespace App\Http\Controllers\Foreground;

use App\Http\Controllers\Controller;
use App\Libraries\Status;
use App\Repositories\OrderRepository;
use App\Services\CMS\OrderService;
use App\Services\Foreground\ProfileOrderService;
use App\ViewModels\Foreground\Profile\Order\DetailShowIndexViewModel;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;

class OrderController extends Controller
{
    public function create()
    {
        return view("foreground.order.create");
    }

    public function index(Request $request)
    {
        Validator::make($request->all(), [
            'orderNumber' => 'required|string',
            'phone' => 'nullable|string',
        ], [
            'orderNumber.required' => '請輸入訂單編號',
            'orderNumber.string' => '訂單編號格式錯誤',
            'phone.string' => '手機號碼格式錯誤',
        ])->validate();

        $result = ProfileOrderService::getDetailPage(
            $request->orderNumber,
            $request->phone,
            null
        );

        if (Status::isFails($result)) {
            return redirect()->back()->with(Status::bladeFails("找不到您要查詢的訂單"));
        }

        $sourceKey = $result['data']['order']->source_key;

        return redirect()->route('foreground.cart.checkoutIndex', [
            "orderNo" => $result['data']['order']->order_no,
        ]);

    }
}
