<?php

namespace App\Http\Controllers\Foreground;

use App\Http\Controllers\Controller;
use App\Libraries\Status;
use App\Services\Foreground\ProductService;
use App\ViewModels\Foreground\Product\GroupBuying\IndexViewModel;
use App\ViewModels\Foreground\Product\ShowViewModel;
use Illuminate\Http\Request;

class ProductGroupBuyingController extends Controller
{
    protected string $rootProductCategoryId = "2";

    public function index(Request $request)
    {
        $result = ProductService::getProductGroupBuyingPage(
            $request->categoryId,
            $this->rootProductCategoryId,
            $request->keyword
        );

        $viewModel = new IndexViewModel(
            $request,
            $result['products'],
            $result['categories'],
            $result['ancestors'],
        );

        return view("foreground.product-group-buying.index", $viewModel);
    }

    public function show(Request $request, $productId)
    {
        $result = ProductService::getPageShow($productId,
            $this->rootProductCategoryId);

        if (Status::isFails($result)) {
            abort(403, "訪問的連結錯誤");
        }

        $product = $result['data']['product'];
        $ancestors = $result['data']['ancestors'];
        $randomProducts = $result['data']['randomProducts'];

        $viewModel = new ShowViewModel(
            $request,
            $result['data']['product'],
            $productId,
            $request->productVariantId ?? $product->variants->first()->id,
            $ancestors,
            $randomProducts,
            "group-buying"
        );

        return view("foreground.product-normal.show", $viewModel);
        // return view("foreground.product-group-buying.show");
    }
}
