<?php

namespace App\Http\Controllers\Foreground;

use App\Http\Controllers\Controller;
use App\Libraries\Status;
use App\Models\Product;
use App\Repositories\ProductRepository;
use App\Services\Foreground\BannerService;
use App\Services\Foreground\CartService;
use App\Services\Foreground\PostService;
use App\Services\Foreground\ProductService;
use App\Services\Foreground\SettingService;
use App\ViewModels\Foreground\Product\IndexViewModel;
use App\ViewModels\Foreground\Product\ShowViewModel;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\Relations\HasOne;
use Illuminate\Http\Request;
use Illuminate\Http\UploadedFile;
use Illuminate\Support\Facades\Storage;

class ProductNormalController extends Controller
{
    protected string $rootProductCategoryId;

    public function __construct()
    {
        $this->rootProductCategoryId = "1";
    }

    public function index(Request $request)
    {
        // 產品
        $result = ProductService::getNormalPage(
            $request->categoryId,
            $this->rootProductCategoryId,
            $request->keyword
        );

        // 效能描述
        $postResult = PostService::getIndexPage(13, 20, 14, null);

        // 案例見證 banner
        $casesBanner = PostService::getShowPage(2, 11, 0);
        // dd($cases['post']->images->toArray());


        $indexViewModel = new IndexViewModel(
            $request,
            $result['products'],
            $result['productCategories'],
            $request->categoryId,
            $result['ancestors'],
            $result['banner'],
            $postResult['posts'],
            $postResult['postCategories'],
            $casesBanner['post']
        );

        return view("foreground.product-normal.index", $indexViewModel);
    }

    public function show(Request $request, string $productId)
    {
        $result = ProductService::getPageShow($productId, $this->rootProductCategoryId);

        if (Status::isFails($result)) {
            abort(404);
        }

        $settings = SettingService::getSettings();

        $product = $result['data']['product'];
        $ancestors = $result['data']['ancestors'];
        $randomProducts = $result['data']['randomProducts'];

        $viewModel = new ShowViewModel(
            $request,
            $settings,
            $result['data']['product'],
            $result['data']['addOn'],
            $productId,
            $request->productVariantId ?? $product->variants->first()->id,
            $ancestors,
            $randomProducts,
            "normal"
        );

        return view("foreground.product-normal.show", $viewModel);
    }
}
