<?php

namespace App\Http\Controllers\Foreground;

use App\Http\Controllers\Controller;
use App\Libraries\Status;
use App\Models\User;
use App\Repositories\UserRepository;
use App\Services\CMS\UserService;
use Illuminate\Auth\Events\PasswordReset;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Password;
use Illuminate\Support\Facades\Route;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Str;

class ProfileController extends Controller
{
    /**
     * 因為沒有會員登入後的 dashboard 概念，因此這裡僅作重新導向
     */
    public function index()
    {
        return redirect()->route("foreground.profile.orders.index");
    }

    public function create()
    {
    }

    public function store(Request $request)
    {
    }

    public function show($id)
    {
    }

    public function normalEdit()
    {
        $user = UserRepository::getUser(["*"], Auth::guard("web")->id());

        return view("foreground.profile.normal.edit", compact('user'));
    }

    public function passwordEdit()
    {
        $user = UserRepository::getUser(["*"], Auth::guard("web")->id());
        return view("foreground.profile.password.edit", compact('user'));
    }

    public function normalUpdate(Request $request, User $user)
    {
        // 驗證表單欄位
        Validator::make($request->all(), [
            'email' => ['required', 'email'],
            'name' => ['required'],
            'gender' => ['required'],
            'birthday' => ['required'],
            'phone' => ['required'],
            'zipCode' => ['required'],
            'county' => ['required'],
            'district' => ['required'],
            'address' => ['required'],
        ], [
            'email.required' => '請填寫 Email',
            'name.required' => '請填寫姓名',
            'gender.required' => '請選擇性別',
            'birthday.required' => '請選擇生日',
            'phone.required' => '請填寫手機',
            'zipCode.required' => '請填寫完整地址',
            'county.required' => '請填寫完整地址',
            'district.required' => '請填寫完整地址',
            'address.required' => '請填寫完整地址',
        ])->validate();


        $result = UserService::updateUserByModel(
            $user,
            $request->name,
            $request->gender,
            $request->birthday,
            $request->phone,
            $request->email,
            $request->zipCode,
            $request->county,
            $request->district,
            $request->address
        );

        return redirect()->back()->with(Status::bladeSuccess("更新成功"));

    }

    public function passwordUpdate(Request $request, User $user)
    {
        Validator::make([
            'old_password' => $request->old_password,
            'password' => $request->password,
            'password_confirmation' => $request->password_confirmation,
        ], [
            'old_password' => [
                'required',
            ],
            'password' => [
                'required',
                'confirmed',
                'min:6',
                'regex: /^(?=.*[a-zA-Z])(?=.*\d).+$/'
            ],
        ], [
            'old_password.required' => '請填寫舊密碼',
            'password.regex' => '請輸入6位(含)以上英文和數字組合密碼',
            'password.required' => '請填寫密碼',
            'password.min' => '密碼最少須要 6 位數',
            'password.confirmed' => '密碼驗證錯誤',
        ])->validate();


        // 用法參考 https://laravel.com/docs/10.x/authentication#confirming-the-password
        if ( ! Hash::check($request->old_password, $user->password)) {
            return redirect()->back()->with(Status::bladeFails("舊密碼錯誤"));
        }

        /**
         * @see ResetPassword::update 更新方式參照
         */
        $user->forceFill([
            'password' => Hash::make($request->password),
        ]);

        $user->save();

        event(new PasswordReset($user));

        return redirect()->back()->with(Status::bladeSuccess("新密碼設置成功"));
    }
}
