<?php

namespace App\Http\Controllers\Foreground;

use App\Http\Controllers\Controller;
use App\Libraries\CartChannel;
use App\Libraries\Status;
use App\Presenters\OrderPresenter;
use App\Services\Foreground\OrderService;
use App\Services\Foreground\ProfileOrderService;
use App\ViewModels\Foreground\Profile\Order\IndexViewModel;
use App\ViewModels\Foreground\Profile\Order\DetailShowIndexViewModel;
use App\ViewModels\Foreground\Profile\Order\GroupBuyIndexViewModel;
use App\ViewModels\Foreground\Profile\Order\LimitTimePreOrderIndexViewModel;
use App\ViewModels\Foreground\Profile\Order\NormalIndexViewModel;
use Illuminate\Support\Facades\Auth;

class ProfileOrderController extends Controller
{
    /**
     * 綜合的商品訂單
     */
    public function index()
    {
        $result = ProfileOrderService::getOrders(
            CartChannel::Normal,
            Auth::guard("web")->id()
        );

        $indexViewModel = new IndexViewModel(
            $result['data']['orders']
        );

        return view("foreground.profile.orders.normal-index",
            $indexViewModel
        );
    }


    /**
     * 一般商品訂單
     */
    public function normalIndex()
    {
        $result = ProfileOrderService::getNormalUserOrders(
            CartChannel::Normal,
            Auth::guard("web")->id()
        );

        $indexViewModel = new NormalIndexViewModel(
            $result['data']['orders']
        );

        return view("foreground.profile.orders.normal-index",
            $indexViewModel);
    }

    /**
     * 拼團預購
     */
    public function groupBuyingIndex()
    {
        $result = ProfileOrderService::getGroupBuyUserOrders(
            CartChannel::GroupBuying,
            Auth::guard("web")->id()
        );


        $indexViewModel = new GroupBuyIndexViewModel(
            $result['data']['orders'],
        );

        return view("foreground.profile.orders.group-buying-index", $indexViewModel);
    }

    /**
     * 限時預購
     */
    public function limitTimePreOrderIndex()
    {
        $result = ProfileOrderService::getLimitTimePreOrderUserOrders(
            CartChannel::LimitTimePreorder,
            Auth::guard("web")->id()
        );


        $indexViewModel = new LimitTimePreOrderIndexViewModel(
            $result['data']['orders'],
        );

        return view("foreground.profile.orders.limit-time-pre-order-index",
            $indexViewModel);
    }


    public function detailsShow(string $orderNo)
    {
        $result = ProfileOrderService::getDetailPage(
            $orderNo,
            null,
            Auth::guard("web")->id()
        );

        if (Status::isFails($result)) {
            abort(404);
        }

        // 目前直接整合導向到購物車第二 & 三步驟
        return redirect()->route("foreground.cart.checkoutIndex", [
            "orderNo" => $orderNo,
        ]);

        // 這裡備著，提供未來須要
        // // 這個明細應該要被共用，可以是會員查看與非會員查看
        // $viewModel = new DetailShowIndexViewModel(
        //     $result['data']['order'],
        //     $result['data']['rangeFormat'],
        //     $result['data']['order']['source_key'],
        //     false
        // );
        // return view("foreground.profile.orders.detail-show", $viewModel);
    }
}
