<?php

namespace App\Http\Controllers\Foreground;

use App\Http\Controllers\Controller;
use App\Libraries\Status;
use App\Services\Foreground\CouponService;
use Endroid\QrCode\Builder\Builder;
use Endroid\QrCode\Encoding\Encoding;
use Endroid\QrCode\ErrorCorrectionLevel\ErrorCorrectionLevelHigh;
use Endroid\QrCode\Label\Alignment\LabelAlignmentCenter;
use Endroid\QrCode\Label\Font\NotoSans;
use Endroid\QrCode\RoundBlockSizeMode\RoundBlockSizeModeMargin;
use Endroid\QrCode\Writer\PngWriter;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;


class QRcodeController extends Controller
{
    /**
     * 指定 data 產生 Qrcode 圖片
     *
     * @param  Request  $request
     * @param  string  $encodeData  要產生的文字資料，請使用 encrypt() 編碼
     *
     * @return string
     */
    public function index(Request $request, string $encodeData)
    {
        $decryptUrl = decrypt($encodeData);

        $result = Builder::create()
            ->writer(new PngWriter())
            ->data($decryptUrl)
            ->size(400)
            ->build();


        // Generate a data URI to include image data inline (i.e. inside an <img> tag)
        $dataUri = $result->getDataUri();

        $html = "<img src='$dataUri' alt='QR Code' />";

        // 添加一個超連結方便開發者點擊
        if (config("app.env") !== "production") {
            $html .= "<br><a href='$decryptUrl' target='_blank'>測試掃描</a>";
        }

        return $html;
    }

    public function couponScanningEdit(Request $request, string $couponCode)
    {
        Validator::make([
            'userId' => $request->userId
        ], [
            'userId' => 'nullable'
        ]);


        $result = CouponService::getCouponByCode($couponCode, $request->userId);
        if (Status::isFails($result)) {
            abort(403, "優惠券編號並不存在");
        }


        return view("foreground.qrcode.edit", [
            'coupon' => $result['data']['coupon'],
            'couponUserRelationship' => $result['data']['couponUserRelationship'],
            'user' => $result['data']['user'],
        ]);
    }


    public function useUpdate(Request $request)
    {
        Validator::make($request->all(), [
            'couponId' => 'required',
            'couponUserRelationshipId' => 'nullable',
            'userId' => 'nullable',
        ]);

        $result = CouponService::updateUsed(
            $request->couponId,
            $request->couponUserRelationshipId,
            $request->userId,
            "qrcodeScanning"
        );

        if (Status::isFails($result)) {
            return redirect()->back()->with(Status::bladeFails($result['message']));
        }

        return redirect()->back()->with(Status::bladeSuccess("優惠券使用成功"));
    }
}
