<?php

namespace App\Http\Controllers\Foreground;

use App\Http\Controllers\Controller;
use App\Libraries\Status;
use App\Models\SettingReservation;
use App\Services\Foreground\FormService;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;

class ReservationController extends Controller
{
    protected array $rules
        = [
            "name" => ["required", "max:30"],
            "phone" => ["required", "max:20"],
            "item-name" => ["required"],
            "date" => ["required"],
            "time" => ["required"],
            "message" => ["nullable"],
        ];

    protected array $messages
        = [
            'name.required' => '請輸入姓名',
            'name.max' => '姓名最多 30 個字',
            'phone.required' => '請輸入手機',
            'phone.max' => '手機最多 20 個字',
            'item-name.required' => '請選擇預約項目',
            'date.required' => '請選擇預約日期',
            'time.required' => '請選擇預約時間',
        ];

    public function create(string $group = "1")
    {
        $settingReservation = SettingReservation::query()->find(1);
        if ($settingReservation === null) {
            abort(403, "預約功能尚未開放");
        }

        return view('foreground.reservation.create', compact('group', 'settingReservation'));
    }

    public function store(Request $request)
    {
        // 驗證欄位
        $make = Validator::make($request->all(), $this->rules, $this->messages);
        if ($make->fails()) {
            return redirect()->route("foreground.reservation.create")
                ->withErrors($make)
                ->withInput();
        }

        FormService::insertReservation(
            $request->name,
            $request->phone,
            $request->{'item-name'},
            $request->date,
            $request->time,
            $request->message,
        );

        return redirect()->route('foreground.reservation.resultIndex')->withInput();
    }

    public function resultIndex(Request $request)
    {
        if ($request->old("name") === null) {
            return redirect()->route("foreground.reservation.create");
        }
        // dd($request->session()->all());
        return view('foreground.reservation.result-index', [
            'request' => $request,
        ]);
    }
}
