<?php

namespace App\Http\Controllers\Foreground;

use App\Http\Controllers\Controller;
use App\Libraries\Status;
use App\Repositories\UserRepository;
use App\Services\Foreground\UserService;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;

/**
 * 註冊系統
 */
class SignupController extends Controller
{
    protected $rules = [
        'email' => ['required', 'email'],
        'password' => [
            'required',
            'confirmed',
            'min:6',
            'regex: /^(?=.*[a-zA-Z])(?=.*\d).+$/'
        ],
        'name' => ['required'],
        'gender' => ['required'],
        'birthday' => ['required'],
        'phone' => ['required'],
        'i-got-it' => ['required']
    ];

    protected $message = [
        'email.required' => '請填寫 Email',
        'password.regex' => '請輸入6位(含)以上英文和數字組合密碼',
        'password.required' => '請填寫密碼',
        'password.min' => '密碼最少須要 6 位數',
        'password.confirmed' => '密碼驗證錯誤',
        'name.required' => '請填寫姓名',
        'gender.required' => '請選擇性別',
        'birthday.required' => '請選擇生日',
        'phone.required' => '請填寫手機',
        'i-got-it.required' => '請點選我已詳細閱讀並同意 《個資保護》'
    ];

    public function index()
    {
    }

    public function create(Request $request)
    {
        return view("foreground.signup.create", [
            'redirect' => $request->redirect ?? ''
        ]);
    }


    public function store(Request $request)
    {
        $validator = Validator::make($request->all(), $this->rules, $this->message)
            // 依然使用預設的重新導向較方便
            ->validate();

        $result = UserService::createSimple(
            $request->email,
            "website",
            $request->password,
            $request->name,
            $request->gender,
            $request->birthday,
            $request->phone,
            $request->phone_captcha,
            null,
            null
        );

        if ($result['status'] === "fails" and $result['code']
            === "emailExists"
        ) {
            return redirect()->back()->with(
                Status::bladeFails("您填寫的 Email 已經被註冊過了，請重新填寫")
            );
        }

        session()->flash("singUpAutoLoginUser", $result['data']['user']);

        return redirect()->route("foreground.signup.loginIndex");
    }

    public function loginIndex()
    {
        $singUpLogin = session()->get('singUpAutoLoginUser');

        if (empty($singUpLogin)) {
            return redirect()->route("foreground.signup.create")
                ->with(Status::bladeFails("操作錯誤"));
        }

        Auth::guard("web")->login($singUpLogin);

        return redirect()->route("foreground.profile.index");
    }

    public function show($id)
    {
    }

    public function edit($id)
    {
    }

    public function update(Request $request, $id)
    {
    }

    public function destroy($id)
    {
    }
}
